/*
 * Copyright 2018 Equilinox Mod Kit Project
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *    the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package equilinoxmodkit.util;


import equilinoxmodkit.EMK;
import main.MainApp;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;


/**
 * Static class that offers easy to use log-writing capabilities.
 *
 * @author EMK Team
 */
public final class Logger {
	
	
	/** File the logged messages get written to when calling {@link #saveLogFile()} */
	private static final File LOG_FILE = new File( "emk.log" );
	
	
	private static boolean firstSave = true;
	private static String tempString = "";
	
	
	/** List of Equilinox Mod Kit classes to determine the message prefix (see {@link #buildPrefix(String)}). */
	private static final String[] EMK_CLASSES = new String[] {
			// EXPERIMENTAL CLASSES
			"equilinoxmodkit.experimental.EmkKeyboard",
			"equilinoxmodkit.experimental.EmkMouse",
			"equilinoxmodkit.experimental.EmkSound",
			"equilinoxmodkit.experimental.EmkTexture",
			// EMK CLASSES
			"equilinoxmodkit.mod.Dependency",
			"equilinoxmodkit.mod.EquilinoxMod",
			"equilinoxmodkit.mod.ModInfo",
			"equilinoxmodkit.util.Logger",
			"equilinoxmodkit.util.VersionUtil",
			"equilinoxmodkit.EMK",
			// EML CLASSES
			"equilinoxmodkit.loader.EmlLauncher",
			"equilinoxmodkit.loader.EmlTweaker",
			"equilinoxmodkit.loader.ModLoader",
			"equilinoxmodkit.mixins.MixinMainApp",
			"equilinoxmodkit.mixins.MixinShaderProgram"
	};
	
	
	/** Private constructor because of statically used class. */
	private Logger() {
	}
	
	
	/**
	 * Writes the specified message(s) with the calling classes name as prefix (see {@link #buildPrefix(String)}) using {@code System.out.println}
	 * to console and to the log file (log file output not yet implemented!).
	 *
	 * @param msg Message(s) to be logged
	 */
	public static void logMsg( Object... msg ) {
		StringBuilder s = new StringBuilder();
		s.append( buildPrefix( Thread.currentThread().getStackTrace()[ 2 ].getClassName() ) );
		
		for( Object o : msg )
			s.append( o );
		
		System.out.println( s );
		tempString += s + "\n";
	}
	
	/**
	 * Writes the specified message(s) with the calling classes name as prefix (see {@link #buildPrefix(String)}) using {@code System.err.println}
	 * to console and with the prefix {@code ERROR:} to the log file (log file output not yet implemented!).
	 *
	 * @param msg Message(s) to be logged
	 */
	public static void logWarning( Object... msg ) {
		StringBuilder s = new StringBuilder();
		s.append( buildPrefix( Thread.currentThread().getStackTrace()[ 2 ].getClassName() ) );
		
		for( Object o : msg )
			s.append( o );
		
		System.err.println( s );
		tempString += "ERROR: " + s + "\n";
	}
	
	/**
	 * Writes all messages that have been logged since last calling this method to {@link #LOG_FILE} and adds information about the date and used
	 * version of Equilinox and the Equilinox Mod Kit if it's the first time this method gets called during this boot.
	 */
	public static void saveLogFile() {
		boolean append = true;
		if(firstSave) {
			append = false;
			firstSave = false;
			tempString = "EMK.log " + getDateString() + "\n"
					+ "Equilinox " + MainApp.VERSION_STRING + "\n"
					+ "EMK Version " + EMK.VERSION + "\n\n" + tempString;
		}
		
		try( BufferedWriter writer = new BufferedWriter( new FileWriter( LOG_FILE,append ) ) ) {
			Logger.logMsg( "Saving log to file..." );
			writer.write( tempString );
			tempString = "";
		} catch( IOException e ) {
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Generates a prefix for the log message. {@code [EMK] [className]} for internal EMK classes and {@code [package.className]} for other classes.
	 *
	 * @param className Name of the calling class
	 *
	 * @return Generated prefix for the to write message.
	 */
	private static String buildPrefix( String className ) {
		if(isEmkClass( className )) {
			String[] tokens = className.split( "\\." );
			return "[EMK] [" + tokens[ tokens.length - 1 ] + "] ";
		} else if(className.equals( "equilinoxmodkit.util.LoggerUtil" )) {
			return ">> ";
		} else {
			return "[" + className + "] ";
		}
	}
	
	/**
	 * Checks if {@code className} matches one of the internal EMK classes to determine the type of log message prefix
	 * (see {@link #buildPrefix(String)}.
	 *
	 * @param className Name of the calling class
	 *
	 * @return {@code True} if {@code className} is an internal EMK class.
	 */
	private static boolean isEmkClass( String className ) {
		for( String s : EMK_CLASSES ) {
			if(className.equals( s ))
				return true;
		}
		return false;
	}
	
	/**
	 * Formats the current date and time to {@code yyyy/MM/dd HH:mm:ss}.
	 *
	 * @return Current date and time.
	 */
	private static String getDateString() {
		return new SimpleDateFormat( "yyyy/MM/dd HH:mm:ss" ).format( new Date() );
	}
	
	
}
